@@ -1,30 +1,32 @@
-# Copyright (C) 2009-2014 Wander Lairson Costa
+# Copyright 2009-2017 Wander Lairson Costa
+# Copyright 2009-2020 PyUSB contributors
 #
-# The following terms apply to all files associated
-# with the software unless explicitly disclaimed in individual files.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-# The authors hereby grant permission to use, copy, modify, distribute,
-# and license this software and its documentation for any purpose, provided
-# that existing copyright notices are retained in all copies and that this
-# notice is included verbatim in any distributions. No written agreement,
-# license, or royalty fee is required for any of the authorized uses.
-# Modifications to this software may be copyrighted by their authors
-# and need not follow the licensing terms described here, provided that
-# the new terms are clearly indicated on the first page of each file where
-# they apply.
+# 1. Redistributions of source code must retain the above copyright
+# notice, this list of conditions and the following disclaimer.
 #
-# IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
-# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
-# ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
-# DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
+# 2. Redistributions in binary form must reproduce the above copyright
+# notice, this list of conditions and the following disclaimer in the
+# documentation and/or other materials provided with the distribution.
 #
-# THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
-# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
-# IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
-# NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
-# MODIFICATIONS.
+# 3. Neither the name of the copyright holder nor the names of its
+# contributors may be used to endorse or promote products derived from
+# this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 from ctypes import *
 import usb.util
@@ -35,14 +37,14 @@ import usb._interop as _interop
 import usb._objfinalizer as _objfinalizer
 import errno
 import math
-from usb.core import USBError
+from usb.core import USBError, USBTimeoutError
 import usb.libloader
 
 __author__ = 'Wander Lairson Costa'
 
 __all__ = [
             'get_backend',
-            'LIBUSB_SUCESS',
+            'LIBUSB_SUCCESS',
             'LIBUSB_ERROR_IO',
             'LIBUSB_ERROR_INVALID_PARAM',
             'LIBUSB_ERROR_ACCESS',
@@ -55,14 +57,14 @@ __all__ = [
             'LIBUSB_ERROR_INTERRUPTED',
             'LIBUSB_ERROR_NO_MEM',
             'LIBUSB_ERROR_NOT_SUPPORTED',
-            'LIBUSB_ERROR_OTHER'
+            'LIBUSB_ERROR_OTHER',
             'LIBUSB_TRANSFER_COMPLETED',
             'LIBUSB_TRANSFER_ERROR',
             'LIBUSB_TRANSFER_TIMED_OUT',
             'LIBUSB_TRANSFER_CANCELLED',
             'LIBUSB_TRANSFER_STALL',
             'LIBUSB_TRANSFER_NO_DEVICE',
-            'LIBUSB_TRANSFER_OVERFLOW'
+            'LIBUSB_TRANSFER_OVERFLOW',
         ]
 
 _logger = logging.getLogger('usb.backend.libusb1')
@@ -269,6 +271,7 @@ def _get_iso_packet_list(transfer):
     return list_type.from_address(addressof(transfer.iso_packet_desc))
 
 _lib = None
+_lib_object = None
 
 def _load_library(find_library=None):
     # Windows backend uses stdcall calling convention
@@ -304,6 +307,10 @@ def _setup_prototypes(lib):
             POINTER(POINTER(c_void_p))
         ]
 
+    # libusb_device *libusb_get_parent (libusb_device *dev)
+    lib.libusb_get_parent.argtypes = [c_void_p]
+    lib.libusb_get_parent.restype = c_void_p
+
     # void libusb_free_device_list (libusb_device **list,
     #                               int unref_devices)
     lib.libusb_free_device_list.argtypes = [
@@ -591,6 +598,8 @@ def _check(ret):
     if ret < 0:
         if ret == LIBUSB_ERROR_NOT_SUPPORTED:
             raise NotImplementedError(_strerror(ret))
+        elif ret == LIBUSB_ERROR_TIMEOUT:
+            raise USBTimeoutError(_strerror(ret), ret, _libusb_errno[ret])
         else:
             raise USBError(_strerror(ret), ret, _libusb_errno[ret])
 
@@ -709,13 +718,22 @@ class _LibUSB(usb.backend.IBackend):
 
     @methodtrace(_logger)
     def _finalize_object(self):
-        self.lib.libusb_exit(self.ctx)
+        if self.ctx:
+            self.lib.libusb_exit(self.ctx)
 
 
     @methodtrace(_logger)
     def enumerate_devices(self):
         return _DevIterator(self.ctx)
 
+    @methodtrace(_logger)
+    def get_parent(self, dev):
+        _parent = self.lib.libusb_get_parent(dev.devid)
+        if _parent is None:
+            return None
+        else:
+            return _Device(_parent)
+
     @methodtrace(_logger)
     def get_device_descriptor(self, dev):
         dev_desc = _libusb_device_descriptor()
@@ -937,12 +955,14 @@ class _LibUSB(usb.backend.IBackend):
         return transferred.value
 
 def get_backend(find_library=None):
-    global _lib
+    global _lib, _lib_object
     try:
         if _lib is None:
             _lib = _load_library(find_library=find_library)
             _setup_prototypes(_lib)
-        return _LibUSB(_lib)
+        if _lib_object is None:
+            _lib_object = _LibUSB(_lib)
+        return _lib_object
     except usb.libloader.LibraryException:
         # exception already logged (if any)
         _logger.error('Error loading libusb 1.0 backend', exc_info=False)
