@@ -1,30 +1,32 @@
-# Copyright (C) 2009-2014 Wander Lairson Costa
+# Copyright 2009-2017 Wander Lairson Costa
+# Copyright 2009-2020 PyUSB contributors
 #
-# The following terms apply to all files associated
-# with the software unless explicitly disclaimed in individual files.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-# The authors hereby grant permission to use, copy, modify, distribute,
-# and license this software and its documentation for any purpose, provided
-# that existing copyright notices are retained in all copies and that this
-# notice is included verbatim in any distributions. No written agreement,
-# license, or royalty fee is required for any of the authorized uses.
-# Modifications to this software may be copyrighted by their authors
-# and need not follow the licensing terms described here, provided that
-# the new terms are clearly indicated on the first page of each file where
-# they apply.
+# 1. Redistributions of source code must retain the above copyright
+# notice, this list of conditions and the following disclaimer.
 #
-# IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
-# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
-# ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
-# DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
+# 2. Redistributions in binary form must reproduce the above copyright
+# notice, this list of conditions and the following disclaimer in the
+# documentation and/or other materials provided with the distribution.
 #
-# THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
-# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
-# IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
-# NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
-# MODIFICATIONS.
+# 3. Neither the name of the copyright holder nor the names of its
+# contributors may be used to endorse or promote products derived from
+# this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 r"""usb.core - Core USB features.
 
@@ -40,8 +42,8 @@ show_devices() - a function to show the devices present.
 
 __author__ = 'Wander Lairson Costa'
 
-__all__ = [ 'Device', 'Configuration', 'Interface', 'Endpoint', 'find',
-            'show_devices' ]
+__all__ = [ 'Device', 'Configuration', 'Interface', 'Endpoint', 'USBError',
+            'USBTimeoutError', 'NoBackendError', 'find', 'show_devices' ]
 
 import usb.util as util
 import copy
@@ -58,10 +60,19 @@ _logger = logging.getLogger('usb.core')
 
 _DEFAULT_TIMEOUT = 1000
 
+_sentinel = object()
+
 def _set_attr(input, output, fields):
     for f in fields:
        setattr(output, f, getattr(input, f))
 
+def _try_getattr(object, name):
+    try:
+        attr = getattr(object, name)
+    except :
+        attr = _sentinel
+    return attr
+
 def _try_get_string(dev, index, langid = None, default_str_i0 = "",
         default_access_error = "Error Accessing String"):
     """ try to get a string, but return a string no matter what
@@ -283,6 +294,14 @@ class USBError(IOError):
         IOError.__init__(self, errno, strerror)
         self.backend_error_code = error_code
 
+class USBTimeoutError(USBError):
+    r"""Exception class for connection timeout errors.
+
+    Backends must raise this exception when a call on a USB connection returns
+    a timeout error code.
+    """
+    pass
+
 class NoBackendError(ValueError):
     r"Exception class when a valid backend is not found."
     pass
@@ -800,6 +819,9 @@ class Device(_objfinalizer.AutoFinalizedObject):
         else:
             self.speed = None
 
+        self._has_parent = None
+        self._parent = None
+
     @property
     def langids(self):
         """ Return the USB device's supported language ID codes.
@@ -841,6 +863,18 @@ class Device(_objfinalizer.AutoFinalizedObject):
             self._product = util.get_string(self, self.iProduct)
         return self._product
 
+    @property
+    def parent(self):
+        """ Return the parent device. """
+        if self._has_parent is None:
+            _parent = self._ctx.backend.get_parent(self._ctx.dev)
+            self._has_parent = _parent is not None
+            if self._has_parent:
+                self._parent = Device(_parent, self._ctx.backend)
+            else:
+                self._parent = None
+        return self._parent
+
     @property
     def manufacturer(self):
         """ Return the USB device's manufacturer string descriptor.
@@ -1245,7 +1279,7 @@ def find(find_all=False, backend = None, custom_match = None, **args):
     def device_iter(**kwargs):
         for dev in backend.enumerate_devices():
             d = Device(dev, backend)
-            tests = (val == getattr(d, key) for key, val in kwargs.items())
+            tests = (val == _try_getattr(d, key) for key, val in kwargs.items())
             if _interop._all(tests) and (custom_match is None or custom_match(d)):
                 yield d
 
